#include<stdio.h>
#include<string.h>
#include<stdlib.h>

#define OK 0
#define WARNING 1
#define CRITICAL 2
#define UNKNOWN 3
#define PERF_LINE_SIZE 2048
#define STATUS_LINE_SIZE 512 

// perfdata structure is a link list node contain current performance data of Sql sever
struct perf_data
{
	char counter_name[100];
	char value[50];
	long int cntr_value;
	struct perf_data *next;
};

// configur_data structure is a link list node contain configuration data for Sql sever
struct configur_data
{
        char counter_name[100];
	char value[50];
        long int cntr_value;
        char usr_def_name[100];
	char unit[30];
	long int warning_value;
        long int critical_value;
	char threshold_type[5];        //whether threshold is minimum threshold value or maximum threshold value
	int status;
        struct configur_data *next;
};

//declaring two lists
struct perf_data *current_data;
struct configur_data *config_data;


int main(int argc, char *argv[])
{
int mysqlstatus=OK, i=0;
char status_line[STATUS_LINE_SIZE];
char statstring[200];
char tempstring[50];
char perf_line[PERF_LINE_SIZE];
char status_command[200]; 
char flush_command[200];
char variable_command[200];
char statusfile[100]="";
char variablefile[100]="";
char basedir[100]="";
char conffile[100]="";
char *stat[]={"OK","WARNING","CRITICAL","UNKNOWN"};
struct configur_data *config_head;
int status1,status2,status3;
long int time_now;
char timenow[20];

if(argc!=6)
{
	printf("######################################################################################\n");
	printf("# Uses:  ./mysql <host> <user_name> <password> <plugin_directory> <config_file_name> #\n");
	printf("# [1] ./mysql            :-  name of the plugin executable (can use full path)       #\n");
	printf("# [2] <host>             :- IP Address of the host where mysql server is             #\n");
	printf("# [3] <user_name>        :- user name of mysql server                                #\n");
	printf("# [4] <password>         :- password of mysql server                                 #\n");
	printf("# [5] <plugin_directory> :- full path of the plugin directory                        #\n");
	printf("# [6] <config_file_name> :- name of config file (just name not full path)            #\n");
	printf("#                                                                                    #\n");
	printf("# eg. /plugins/MYSQL/mysql 127.0.0.1 admin adminpasswd /plugins/MYSQL mysql.conf     #\n");
	printf("######################################################################################\n");
	exit(2);
}

time_now=time(NULL);
sprintf(timenow,"%ld", time_now);

strcpy(basedir,argv[4]);
strcpy(statusfile,basedir);
strcat(statusfile,"/status");
strcat(statusfile, timenow);
strcat(statusfile, ".out");

strcpy(variablefile,basedir);
strcat(variablefile,"/variable");
strcat(variablefile, timenow);
strcat(variablefile, ".out");

strcpy(conffile,basedir);
strcat(conffile,"/");
strcat(conffile,argv[5]);


//Building Status_command
strcpy(status_command,"/usr/bin/mysql -h "); 
strcat(status_command,argv[1]);
strcat(status_command," -e \"show status\" -u ");
strcat(status_command,argv[2]);
strcat(status_command," --password=");
strcat(status_command,argv[3]);
strcat(status_command," > ");
strcat(status_command,statusfile);


//Building flush_command
strcpy(flush_command,"/usr/bin/mysql -h ");
strcat(flush_command,argv[1]);
strcat(flush_command," -e \"flush status\" -u ");
strcat(flush_command,argv[2]);
strcat(flush_command," --password=");
strcat(flush_command,argv[3]);

//Building variable_command
strcpy(variable_command,"/usr/bin/mysql -h ");
strcat(variable_command,argv[1]);
strcat(variable_command," -e \"show variables\" -u ");
strcat(variable_command,argv[2]);
strcat(variable_command," --password=");
strcat(variable_command,argv[3]);
strcat(variable_command," > ");
strcat(variable_command,variablefile);

//Executing status_command
status1=system(status_command);

//Executing flush_command
status2=system(flush_command);

//Executing variable_command
status3=system(variable_command);

if(status1)
{
	printf("Error Executing show status command %d",status1);

	//remove output files
	strcpy(status_command,"rm -f ");
	strcat(status_command,statusfile);
	system(status_command);

	strcpy(variable_command,"rm -f ");
	strcat(variable_command,variablefile);
	system(variable_command);

	return 2;
}

if(status2)
{
        printf("Error Executing flush status command");

	//remove output files
	strcpy(status_command,"rm -f ");
	strcat(status_command,statusfile);
	system(status_command);

	strcpy(variable_command,"rm -f ");
	strcat(variable_command,variablefile);
	system(variable_command);

        return 2;
}

if(status3)
{
        printf("Error Executing variable status command");

	//remove output files
	strcpy(status_command,"rm -f ");
	strcat(status_command,statusfile);
	system(status_command);

	strcpy(variable_command,"rm -f ");
	strcat(variable_command,variablefile);
	system(variable_command);

        return 2;
}

tempstring[0]='\0';
statstring[0]='\0';
// Initialize two lists
current_data=NULL;
config_data=NULL;
config_head=NULL;
strcpy(status_line , "");

make_perfdata_list();

make_configdata_list(conffile);

compare_two_lists();

config_head = config_data;
while(config_head)
{
	if((config_head->status > mysqlstatus)&& (config_head->status != UNKNOWN))
		mysqlstatus = config_head->status;
	config_head = config_head->next;
}

strcat(status_line,"MySQL: ");
strcat(status_line,stat[mysqlstatus]);
strcat(status_line,"- ");

config_head = config_data;
while(config_head)
{
        if(!strcmp(config_head->threshold_type,"NA"))
	{
		tempstring[0]='\0';
        	statstring[0]='\0';
        	
                strcat(statstring,config_head->usr_def_name);
		strcat(statstring,"=");
                strcat(statstring,config_head->value);
		             
                if(strcmp(config_head->unit,"NA"))
		{
			strcat(statstring,config_head->unit);
		}

		strcat(statstring,"; ");
                
               	if((strlen(status_line)+ strlen(statstring))<STATUS_LINE_SIZE)
                {
                       	strcat(status_line,statstring);
                }
        	
	}
        config_head=config_head->next;
}

config_head = config_data;
while(config_head)
{
        if(strcmp(config_head->threshold_type,"NA"))
	{
		tempstring[0]='\0';
        	statstring[0]='\0';

		if(config_head->status==CRITICAL)
        	{
			strcat(statstring,config_head->usr_def_name);
			strcat(statstring,"=");
                	sprintf(tempstring,"%ld",config_head->cntr_value);
			strcat(statstring,tempstring);
			if(strcmp(config_head->unit,"NA"))
			{
				strcat(statstring,config_head->unit);
			}
			strcat(statstring,"; ");

                	if((strlen(status_line)+ strlen(statstring))<STATUS_LINE_SIZE)
                	{     
                        	strcat(status_line,statstring);
                	}
       		}
	}
        config_head=config_head->next;
}

config_head = config_data;
while(config_head)
{
        if(strcmp(config_head->threshold_type,"NA"))
	{
		tempstring[0]='\0';
        	statstring[0]='\0';

        	if(config_head->status==WARNING)
        	{
                	strcat(statstring,config_head->usr_def_name);
			strcat(statstring,"=");
                	sprintf(tempstring,"%ld",config_head->cntr_value);
               		strcat(statstring,tempstring);
			if(strcmp(config_head->unit,"NA"))
			{
                		strcat(statstring,config_head->unit);
			}
                	strcat(statstring,"; ");
                	if((strlen(status_line)+ strlen(statstring))<STATUS_LINE_SIZE)
                	{
                        	strcat(status_line,statstring);
                	}
        	}
	}
        config_head=config_head->next;
}

config_head = config_data;
while(config_head)
{
	if(strcmp(config_head->threshold_type,"NA"))
	{
        	tempstring[0]='\0';
        	statstring[0]='\0';

        	if((config_head->status!=CRITICAL)&&(config_head->status!=WARNING))
        	{
                	strcat(statstring,config_head->usr_def_name);
			strcat(statstring,"=");
                	sprintf(tempstring,"%ld",config_head->cntr_value);
                	strcat(statstring,tempstring);
			if(strcmp(config_head->unit,"NA"))
			{
                		strcat(statstring,config_head->unit);
                	}
			strcat(statstring,"; ");
                	if((strlen(status_line)+ strlen(statstring))<STATUS_LINE_SIZE)
                	{
                        	strcat(status_line,statstring);
                	}
        	}
	}
        config_head=config_head->next;
}

strcpy(perf_line,"");
strcpy(tempstring,"");
strcpy(statstring,"");
config_head = config_data;
while(config_head)
{
        if(strcmp(config_head->threshold_type,"NA"))
	{
		tempstring[0]='\0';
       		statstring[0]='\0';

        	strcpy(statstring,config_head->usr_def_name);
       		strcat(statstring,"=");

       		sprintf(tempstring,"%ld",config_head->cntr_value);
       		strcat(statstring,tempstring);
		if(strcmp(config_head->unit,"NA"))
		{
			strcat(statstring,config_head->unit);
		}
		strcat(statstring,";");
	
		if(strcmp(config_head->threshold_type,"NTRQ"))
		{
			sprintf(tempstring,"%ld",config_head->warning_value);
        		strcat(statstring,tempstring);
			strcat(statstring,";");
	
			sprintf(tempstring,"%ld",config_head->critical_value);
        		strcat(statstring,tempstring);
        		strcat(statstring,";");
		}
	
		if((strlen(perf_line)+ strlen(statstring))<PERF_LINE_SIZE)
       		{
			strcat(perf_line,statstring);
       			strcat(perf_line," ");
		}
	}
        config_head=config_head->next;
}
		
//remove output files
strcpy(status_command,"rm -f ");
strcat(status_command,statusfile);
system(status_command);

strcpy(variable_command,"rm -f ");
strcat(variable_command,variablefile);
system(variable_command);

printf("%s|%s\n",status_line,perf_line);

return mysqlstatus;
}

int make_perfdata_list()
{
        FILE *fp;
        char line[500];
        char *str;
        //declaring the nodes that will append to the end of list
	//struct perf_data perf_append;
	struct perf_data *temp,*previous;
	
//For Show Status output 
	temp=NULL;
	previous=NULL;
	
	fp =fopen("/opt/pmpplug/libexec/mysql/status.out","r");
	if(fp==NULL)
        {
                printf("status.out:file not found");
                exit(1);
        }
	
	while(fgets(line,sizeof(line),fp)!=NULL)
	{
		
		temp=malloc(sizeof(struct perf_data));
		
		str = strtok(line, "\t ");
		if(str!=NULL)
		{	
			strcpy(temp->counter_name,str);
		}
		else
		{
			strcpy(temp->counter_name,"NULL");
		}
		
		str = strtok(NULL, " \n");
		if(str!=NULL)
		{
			strcpy(temp->value,str);
		}
		else
		{
			strcpy(temp->value,"NULL");
		}
               	
		temp->cntr_value= atol(temp->value);
								
			
		if(current_data==NULL)
                {
			temp->next=NULL;
			current_data=temp;
			previous=temp;
                }
		else
		{
			previous->next=temp;
			previous=temp;
			temp->next=NULL;
		}
 
		
	}

//For variable output
     	
        fp =fopen("/opt/pmpplug/libexec//mysql/variable.out","r");
        if(fp==NULL)
        {
                printf("variable.out:file not found");
                exit(1);
        }

        while(fgets(line,sizeof(line),fp)!=NULL)
        {

                temp=malloc(sizeof(struct perf_data));

                str = strtok(line, "\t ");
		if(str!=NULL)
                {
	                strcpy(temp->counter_name,str);
		}
		else
		{
			strcpy(temp->counter_name,"NULL");
		}
		

                str = strtok(NULL, " \n");
		if(str!=NULL)
		{
			strcpy(temp->value,str);
		}
		else
		{
			strcpy(temp->value,"NULL");
		}

                temp->cntr_value= atol(temp->value);

                if(current_data==NULL)
                {
                        temp->next=NULL;
                        current_data=temp;
                        previous=temp;
                }
                else
                {
                        previous->next=temp;
                        previous=temp;
                        temp->next=NULL;
                }


        }

	
return 0;
}

int make_configdata_list(char *conffile)
{
        FILE *fp;
        char line[500];
        char *str;
        //declaring the nodes that will append to the end of list
        //struct perf_data perf_append;
        struct configur_data *temp,*previous;

        temp=NULL;
        previous=NULL;

        fp =fopen(conffile,"r");
        if(fp==NULL)
        {
                printf("%s:file not found\n",conffile);
                exit(1);
        }



        while(fgets(line,sizeof(line),fp)!=NULL)
        {
	    if(line[0]=='#')
	    {

                temp=malloc(sizeof(struct configur_data));
                
		str = strtok(line+1, "#");
                strcpy(temp->counter_name,str);
                
		str = strtok(NULL, "#");
                temp->warning_value= atol(str);
                        
                str = strtok(NULL, "#");
                temp->critical_value=atol(str);
                        
		str = strtok(NULL, "#");
                strcpy(temp->threshold_type,str);

		str = strtok(NULL, "#");
                strcpy(temp->usr_def_name,str);
			
		str = strtok(NULL, "#");
                strcpy(temp->unit,str);

		temp->status=UNKNOWN;
		temp->cntr_value=-1;           

                if(config_data==NULL)
                {
                        temp->next=NULL;
                        config_data=temp;
                        previous=temp;
                }
                else
                {
                        previous->next=temp;
                        previous=temp;
                        temp->next=NULL;
                }

	}


        }

return 0;
}

int compare_two_lists()
{
	struct perf_data *perf_temp;
	struct configur_data *config_temp;
	perf_temp=current_data;
	config_temp=config_data;
	if(config_temp==NULL)
        return(1);

	while( config_temp!=NULL)
	{
		if(perf_temp==NULL)
		{
			printf("The counter %s not found",config_temp->counter_name);
			config_temp=config_temp->next;
			perf_temp=current_data;
			continue;
		}

		if(strcmp(perf_temp->counter_name,config_temp->counter_name))
		{
			perf_temp=perf_temp->next;
			continue;
		}
				
		if(!strcmp(config_temp->threshold_type,"INCR"))
		{
			if(perf_temp->cntr_value>=config_temp->critical_value)
			{
				config_temp->status=CRITICAL;
			}
			else if(perf_temp->cntr_value>=config_temp->warning_value)
			{
				config_temp->status=WARNING;
			}
			else
			{
				config_temp->status=OK;
                       	}
		
			config_temp->cntr_value=perf_temp->cntr_value;
                        
		}
		else if(!strcmp(config_temp->threshold_type,"DECR"))
		{
			 if(perf_temp->cntr_value<=config_temp->critical_value)
                        {
                                config_temp->status=CRITICAL;
                        }
                        else if(perf_temp->cntr_value<=config_temp->warning_value)
                        {
                                config_temp->status=WARNING;
                        }
                        else
                        {
                                config_temp->status=OK;
                        }

			config_temp->cntr_value=perf_temp->cntr_value;		        
		}

		else if(!strcmp(config_temp->threshold_type,"NTRQ"))
		{
			config_temp->status=OK;
			config_temp->cntr_value=perf_temp->cntr_value;
		}
		else if(!strcmp(config_temp->threshold_type,"NA"))
                {
                        config_temp->status=OK;
                        config_temp->cntr_value=0;
                }
		else
		{
			printf("The Threshold type of %s is not correct",config_temp->counter_name);
			return(1);
			config_temp->cntr_value=perf_temp->cntr_value;
                        config_temp->status=UNKNOWN;
		}

		strcpy(config_temp->value,perf_temp->value);
		config_temp=config_temp->next;
		perf_temp=current_data;
	}
return 0;
}

